<?php

//----------------------------------------------------------------------------
//
// All software and media in this folder and its subdirectories 
// is licensed as open source and creative commons respectively.
//
// Use it, make it better, and let others do the same. 
//
// Copyright (C) 2010  Adam Kumpf (akumpf@teague.com) and Matt Wolfe
// Created as an internal project with Teague Labs (http://labs.teague.com)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//----------------------------------------------------------------------------
	
# Logging in with Google accounts requires setting special identity, so this example shows how to do it.
require_once(dirname(__FILE__).'/openid.php');
require_once(dirname(__FILE__).'/../serverSettings.php');
	
try {
    if(!isset($_GET['openid_mode'])) {
        if(isset($_GET['login'])) {
            $openid = new LightOpenID;
			if(isset($_POST['openid_identifier'])){
				if(stripos($_POST['openid_identifier'], "@gmail.com") > 0){
					// gmail address
					$openid->identity = 'https://www.google.com/accounts/o8/id';
				}else{
					// normal openID
					$openid->identity = $_POST['openid_identifier'];
				}
			}else{
				echo "No openid_identifier was specified. Please try again.";
			}
			$openid->required = array('namePerson', 'namePerson/first', 'namePerson/last', 'contact/email', 'pref/language');
			header('Location: ' . $openid->authUrl());
        }else{
			echo "This OpenID Login script is usually called from another page.<br/>For debugging purposes, you can use this form to attempt logging in.<br/><br/>Upon success, you will be shown your userID which can be used for access control.<br/><br/>";
			echo "<form action='./openidLogin.php?login&stayOnPage' method='post'>";
			echo "OpenID email: <input type='text' name='openid_identifier' /> <button>Submit</button>";
			echo "</form>";
			exit;
		}
    } elseif($_GET['openid_mode'] == 'cancel') {
        echo 'User has canceled authentication!';
    } else {
        $openid = new LightOpenID;
		$openid->required = array('contact/email');
		if(!$openid->validate()){
			echo "User has not logged in.";
		}else{
			$attribs= $openid->getAttributes();
			$email = isset($attribs['contact/email']) ? $attribs['contact/email'] : 'unknown';
			$fname = isset($attribs['namePerson/first']) ? $attribs['namePerson/first'] : '';
			$lname = isset($attribs['namePerson/last']) ? $attribs['namePerson/last'] : '';
			$fullname = isset($attribs['namePerson']) ? $attribs['namePerson'] : '';
			if($fullname == ''){
				$fullname = $fname." ".$lname;
			}
			$expTime = time()+60*60*24*30; // 30 days
			setcookie('login', $openid->identity.','.md5($openid->identity.$SECRET_WORD), $expTime, $COOKIE_PATH); // openID is 'salted' for verification later.
			setcookie('fullname', $fullname, $expTime, $COOKIE_PATH);
			setcookie('email', $email, $expTime, $COOKIE_PATH);
			if(!isset($_GET['stayOnPage'])){
				header('Location: ' ."../".$AFTER_LOGIN_PAGE); // redirect to post-login page.
			}else{
				echo "OpenID login was successful.<br/><br/><table style='border: 1px solid #CCCCCC;'>";
				echo "<tr><td style='color: #66AA66;'>User ID</td><td>" . $openid->identity . "</td></tr>";
				echo "<tr><td style='color: #66AA66;'>md5 ID</td><td>" . md5($openid->identity) . "</td></tr>";
				echo "<tr><td style='color: #66AA66;'>Email</td><td>" . $email . "</td></tr>";
				echo "<tr><td style='color: #66AA66;'>Name</td><td>" .  $fullname . "</td></tr>";
				echo "</table><br/><br/>";
			}
		}
    }
} catch(ErrorException $e) {
    echo $e->getMessage();
}

?>